;sntp_time, MB03. Updates RTC on MB03+ via Wifi module
;Hood
;-ver 0.4.	28.3.2021, version purely in machine code, aimed for MB03+ setup
;works as standalone utility. Same functionality as ver 0.5. Preserves user's CPU speed
;(ZX Element users only).
;- ver 0.3	addind Wifi reset (AT+RST command)
;tuto verzi dvme k ledu, je v n rozprcovna nezvislost na BASICu (INPUT ze strojku)
;a tak reset Wifi, jdeme dlat verzi 0.4 kter bude jen strojk, umstn v setupu MB
;BASIC k 0.3. je v adresi v TAP
;samostan fungujc utilita ve strojku. Zvisl ovem na posunu GMT, kter se te z MB03+
;- ver 0.2	adding CZ servers (GMT +1 by default) 
;		list of CZ SNTP servers: http://phoenix.inf.upol.cz/~bazgierv/ntp.html
;- ver 0.1	winter/summer time manually

	output "sntp_rtc"

;set C=		SCF
;set NC=	AND A, if Z needs to be preserved, then SCF:CCF

;overview:
;SENDING
;1) send_string:	mainly to send direct AT commands
;2) send_data:		to send any type of data


;RECEIVING
;7) receive_data_timeout receive data with timeout

;PARSING
;10) look_for_string:	seeks certain string in memory
;11) look_on_fly:	seeks certain string in the incoming flow of bytes


;OTHER
;16) ConvRStr16		converts ASCII to HL (16 bit)
;17) HL2ASCII		converts HL to ASCII (16 bit)
;18) wait routines	wait certain amount of time (user definable)
;29) send_at_check_ok	sends AT command and checks for answer OK
;40) set_print_speed	sets speed and print it
;48) convASCII2hex	convert eg. "14" to #14



;	DEFINE border_colour	;0=no colours,1=show border colours, yellow for disk operations,
				;violet= communication with module
	
buf_len		equ #8000	;nemelo by byt pevne, ale PHP script to ma tez pevne dano
;buf_len	equ #800
main_port	equ #203B
wifi_port	equ #213b
basic_end	equ #63E0
speed_pos	equ #4300



bios		equ	#FE00
gver_inst	equ	bios+0
wifi_on		equ	bios+4
wifi_off	equ	bios+6
set_baud	equ	bios+8
clear_rece_buf	equ	bios+12
rece_byte	equ	bios+16
send_byte	equ	bios+18
rece_block	equ	bios+20
send_block	equ	bios+22


port_rtc_select	equ	#703B


;Error reports
	;BC=1=send_string_error

	;BC=2=ntp server not responding within timeout
	;BC=3=wifi not connected to AP


	;BC=4=":" not found in raw time&date data
	;BC=7= correct time zone not activated

	;BC=5=AT_RST command not sent
	;BC=6=AT_RST "WIFI DISCONNECT" not returned
	;BC=8=AT+CIPSNTPCFG command not accepted
	
	;BC=27=speed management: AT+UART command not sent
	;BC=28=speed management: AT command not accepted
	;BC=29=speed management: all speeds tested, timeout, fix it in terminal
	;BC=30=speed management: failed, probably Wifi module wrongly set,fix in terminal



		org #E400
;		org bios-len			;






start_here	di
		ld	hl,bios_start
		ld	de,bios
		ld	bc,bios_len
		ldir
		call	bios

		ld	(stack_pointer+1),sp
		ld	sp,0

		ld	bc,#783B		;read out CPU speed and save it later
		ld	a,#30
		out	(c),a
		inc	b
		in	a,(c)			;speed is 0-3
		ld	(return_speed+1),a


		ld	ix,flag1

;		ld	a,(23670)
;		rra
;		rra
;		res	1,(ix+0)
;		jr	nc,st1
;		set	1,(ix+0)	;carry out AT+RST?		
			


;flag1 (IX+0),bit 7=used in "HL2ASCII"
;bit 0=used in "look_for_string"
;bit 1=AT+RST comamnd,0=no, 1=yes	
;bit 2=input/zone shift=0=plus,1=minus
;flag2 (IX+3).bit 0=print send command,0=NO, 1=YES
;bit 4=uart speed, 0=max speed 1152000 or 115200, 1=max speed 3456000

						

st1		ld	hl,16384
		ld	de,16385
		ld	bc,6144
		ld	(hl),0
		ldir
		ld	bc,768
		ld	(hl),7
		ldir
		ld	de,title
		ld	hl,#4005
		call	PRINTHL
		ld	hl,#4105
		ld	b,title_len
underl		ld	a,34			;""""
		push	bc
		call	CHAR
		pop	bc
		djnz	underl		

		ld	hl,speed_pos		;print "Speed" text
		ld	de,speed
		call	PRINTHL
		ld	hl,#4000
		ld	(HLKO+1),hl

		bit	0,(ix+3)
		ld	hl,#54E5	;"push hl:ld d,h"
		jr	nz,printyesno1
		ld	hl,#0518	;"jr ss2"
printyesno1	ld	(send_string),hl




;speed management starts here
;the principle: find out max speed according to FW, then turn on wifi. Find out if UART+Wifi ;communicate, if timeout exceeded, then error. Otherwise establish connection betwee uart and wifi at any speed. ;Then set optimal speed for the FW.

speed_management
		ld	bc,#783B		;set CPU to 3,5 MHz
		ld	a,#30
		out	(c),a
		inc	b
		xor	a
		out	(c),a

		call	gver_inst		;best bios speeds are: 0=115200, 12=1152000 and
		ex	de,hl			;16=3456000
		ld	de,core			;find out core version
		push	de
		ld	bc,3
		ldir
		pop	hl

		ld	d,0			;now set speed and retry answer from AT command
		ld	a,(hl)			;HL=core version found
		cp	20
		jr	c,got_max_speed		;found core <2019xxxxx,core0
		jr	z,sc1
		ld	d,16
		jr	nc,got_max_speed	;NC=found core>2020=contains high speeds (core2)
sc1		inc	hl
		ld	a,(hl)
		cp	5
		jr	c,got_max_speed		;found core 202004xx,core0, max speed 115200
		ld	d,12
		jr	z,got_max_speed		;Z=found core 202005xx, core1, max speed 1152000

		bit	4,(ix+3)
		jr	z,sc2
		ld	d,16
sc2		cp	6			;test speeds max to 12
		jr	nc,got_max_speed	;found core >202006xx, core2,max speed 3456000
		ld	d,0			;speed not found in FW, set it to 0 (115200)

got_max_speed	ld	(ix+8),d		;d=0 or 12 or 16

		ld	(ix+6),#90		;timeout for more routines below. #90 should be enough.
		res	4,(ix+3)		;limit max speed to 1152000 (ver 0.34 cannot transfer at higher 							;speed)
		ld	(ix+5),0		;starting speed in bios HW is 0	(115200 B)

		call	wifi_on
;		call	wait_short

;		and	a			;not needed at the moment, just in case we want
;		call	set_baud		;to set original speed
;		ld	(ix+7),d		;d=speed set currently in HW


		call	send_at_check_ok
		jr	z,correct_speed
speed_loop	call	clear_rece_buf		;good if wifi turned on from BASIC

		ld	hl,(HLKO+1)
		push	hl
		ld	hl,#4600
;		push	hl
		ld	de,neg_wifi
		call	PRINTHL
;		pop	hl
;		ld	de,spaces
;		call	PRINTHL
		pop	hl
		ld	(HLKO+1),hl

		call	switch_speed		;answer not found, we should change speeds and retest
		jr	z,correct_speed		;Z=UART+wifi communicate

		dec	(ix+6)
		ld	bc,29
		jp	z,error			;timeout run out, error
		jr	speed_loop			
						;tady odlisit, zd auz byla wifi zapnuta, nebo ne


correct_speed	and	a			;answer found, uart& module communicate
		call	set_baud		;is it the best speed we want?
		ld	a,(ix+8)
		cp	d
		jp	z,wifi_cont		;Z speed found, we want 0 or 12 for ver 034
						;NZ not the one we want, so set it in, correct in A

;core=0-7, core1=0-12, core2=23 (limited to 12 in ver 034)
set_best_speed					;d=speed set currently in HW, a=speed we want

		ld	de,TX_buf		;now prepare AT comand to TX_buf
		push	de
		ld	hl,uart
		ld	bc,12
		ldir
		ld	c,6
		ld	hl,_1152000
		or	a
		jr	z,sbs4
		inc	c
		cp	12
		jr	z,sbs4
		bit	4,(ix+3)
		jr	z,sbs4			;speed limited to 12
		ld	hl,_3456000
sbs4		ldir
		ld	hl,_8100
		ld	bc,10
		ldir		
		ex	de,hl
		ld	(hl),0

		pop	hl
;		ld	bc,31
		push	af
;		call	send_data		;set speed on the module via AT+UART+CUR command
		call	send_string
		ld	bc,27
		jp	nz,error	

		pop	af
		or	a
		ld	d,7
		jr	z,sbs1
		ld	d,12
		cp	d
		jr	z,sbs1
		bit	4,(ix+3)
		jr	z,sbs1			;limit max speed to 12
		ld	d,16
		cp	d
		jp	nz,error
		scf
sbs1		ld	a,d
		call	set_print_speed		;set also uart speed

;		ld	b,3
;		call	wait_long_def		;b=3 is ok, 2=not ok (kratit bez ld b,3)


sbs2		call	clear_rece_buf		;timeout would be much better?

		ld	hl,#4600
		ld	de,neg_wifi
		call	PRINTHL

		call	send_at_check_ok
		jr	z,wifi_cont

		ld	hl,#4600
		ld	de,spaces
		call	PRINTHL


		dec	(ix+6)
		jr	nz,sbs2

		ld	hl,#4600
		ld	de,neg_wifi
		call	PRINTHL
		ld	de,wifi_no_success
		call	PRINT
		ld	bc,30
		jp	error		


switch_speed	xor	a			;try out every speed if you find connection UART+Wifi
;		scf				;start with lowest speed, end with 23 (not 7, 12 or 23)
sws2		push	af			;natvrdo zkousim 23 rychlosti, melo by to byt dle fw v MBcku
;		call	set_baud
		call	set_print_speed
;		call	wait_short		;when not here, it speeds up conenction process significantly

;		ld	hl,(HLKO+1)		;this part disables to connect for 1st time
;		push	hl
;		ld	hl,#4600
;		push	hl
;		ld	de,neg_wifi
;		call	PRINTHL
;		pop	hl
;		ld	de,spaces
;		call	PRINTHL
;		pop	hl
;		ld	(HLKO+1),hl


		call	send_at_check_ok
		pop	de
		ret	z			;Z=answer found, D= speed set in HW
		ld	a,d
		inc	a
		cp	24
		jr	c,sws2
		inc	a			;NZ
		ret

;		pop	de
;		ret	z			;Z=answer found, D= speed set in HW
;		ld	a,(ix+8)		;a=0 or 12 or 16. 0=7, 12=12, 16=23
;		or	a
;		ld	e,7
;		jr	z,sws1
;		ld	e,23
;		cp	16
;		jr	z,sws1
;
;sws1		ld	a,e
;		inc	d
;		cp	a
;		ret	nz			;all speeds tested. Connection UART+Wifi not found.
;		ld	a,d
;		jr	sws2		
;end of speed management here




wifi_cont
		and	a
		call	print_speed		;prints speeds 7, 12 or 16 (disabled in ver. 0.33)

		ld	hl,ate0
		call	send_string
		ld	hl,ok
		call	look_on_fly
		;test for answer should be here?



;		ld	b,9			;wait 9 experimentally tested, is ok
;w_cont1	call	wait_long_def		;timeout would be much better?



w_cont1		ld	hl,(HLKO+1)
		push	hl
		ld	hl,#4600
;		push	hl
		ld	de,neg_wifi
		call	PRINTHL
;		pop	hl
;		ld	de,spaces
;		call	PRINTHL
		pop	hl
;		ld	(HLKO+1),a


		call	clear_rece_buf
		ld	hl,cipstatus		;find out, if module connected to wifi
		call	send_string
		ld	bc,1
		jp	nz,error



		ld	hl,RX_buf		;receive the answer about STATUS to RX_buf
		ld	bc,20
;		ld	bc,30
		push	hl
		push	bc
		call	receive_data_timeout

;		ld	hl,(HLKO+1)
;		push	hl
;		ld	hl,#4600
;		push	hl
;		ld	de,neg_wifi
;		call	PRINTHL
;		ld	hl,(HLKO+1)
;		ld	(jr1+1),hl
;		pop	hl
		ld	hl,#4600
		ld	de,spaces
		call	PRINTHL
;		pop	hl
		ld	hl,(HLKO+1)
		ld	(jr1+1),hl

		pop	bc
		pop	hl
		ld	de,STATUS
		call	look_for_string
		ld	h,d
		ld	l,e
		ld	de,answ_5
		call	look_for_string
jr1		ld	hl,#5555
		jr	nz,w_cont2		;NZ=status "5" not found, wifi is connected to AP
		dec	(ix+6)
		jr	nz,w_cont1
;		jr	w_cont3


/*
w_cont2		push	af		
		call	clear_rece_buf
		ld	de,wifi
		call	PRINT
		pop	af
		jr	nz,jr1
*/


w_cont3		ld	hl,#4600
		ld	de,neg_wifi
		call	PRINTHL
		ld	de,wifi_nc_ap
		call	PRINTHL
		ld	bc,3
		jp	error


w_cont2		ld	de,ok_
		call	PRINTHL
		ld	hl,#4000
		ld	(HLKO+1),hl


;		ld	de,connected
;		call	PRINT
;		ld	de,to_AP
;		call	PRINT

;		bit	7,(ix+0)		;seems, server needs a bit of rest before downloading
;		jr	z,main			;Z=Wifi already was ON, make no pause
;		call	wait_short		;do not like it, but test if server needs a little rest

		ld	hl,(HLKO+1)
		push	hl
		ld	hl,#4600
		ld	de,neg_wifi
		call	PRINTHL
		pop	hl
		ld	(HLKO+1),hl


/*					;reset wifi zatim odlozime
reset_wifi	bit	1,(ix+0)
		jr	z,begin_rtc
		ld	hl,#4800
		ld	de,wifi_reset
		call	PRINTHL
		ld	hl,at_rst
		call	send_string
		ld	hl,ok
		call	look_on_fly
		ld	bc,5
		jp	nz,error
;		ld	hl,disconnect
;		call	look_on_fly
;		ld	bc,6
;		jp	nz,error
		

		res	1,(ix+0)
		ld	b,20
resw		call	clear_rece_buf
		djnz	resw
;		call	send_at_check_ok
;		jr	nz,resw

		jp	speed_management
*/



begin_rtc	ld	(ix+6),#70+#30		;timeout for sntp server+cfg activation
		res	2,(ix+0)		;zone is plus

		ld	hl,at_cipsntpcfg	;place GMT shift to the command
		ld	de,RX_buf
		push	de
		ld	bc,16
		ldir
		inc	hl
		push	hl
		ld	h,c

		ld	bc,port_rtc_select	;find out GMT zone from MB setup
		ld	a,#40
		out	(c),a
		inc	b
		in	a,(c)			;A=byte of GMT shift
		or	a
		jp	p,gmt_plus
;		cpl
		neg
		set	2,(ix+0)		;zone is minus

		ex	de,hl
		ld	(hl),"-"
		inc	hl
		ex	de,hl

;		ld	a,"-"
;		ld	(de),a
;		inc	de

gmt_plus	ld	l,a
		call	HL2ASCII
						;shift at place
		pop	hl
		ld	bc,50
		ldir		
		ex	de,hl
		ld	(hl),13
		inc	hl
		ld	(hl),10
		inc	hl
		ld	(hl),b			;comamnd at place

		ld	hl,at_cwmode		;send AT commands to get time&date
		call	send_string
		ld	hl,ok
		call	look_on_fly
		ld	bc,1
		jp	nz,error



;		ld	hl,RX_buf
		pop	hl
		push	hl
		call	send_string		;send AT CFG command from RXbuf

;		ld	hl,at_cipsntpcfg
;		call	send_string

		ld	hl,ok
		call	look_on_fly
		ld	bc,8
		jp	nz,error

		ld	de,gmt_shift		;print GMT shift
		ld	hl,#4400
		call	PRINTHL

		pop	hl			;HL=RXbuf
		ld	bc,16
		add	hl,bc
		ld	de,block_asci
		push	de
		ld	a,(hl)
		cp	"-"
		jr	z,gmts2
		ld	a,"+"
		ld	(de),a
		inc	de
gmts2		ld	bc,3
gmts1		ldi
		ld	a,(hl)
		cp	","
		jr	nz,gmts1
		xor	a
		ld	(de),a
		pop	de
		call	PRINT
		


activate_tz	ld	hl,at_cipsntpcfg?
		call	send_string
		ld	bc,buf_len
		ld	hl,RX_buf
		push	hl
		call	receive_data_timeout
		pop	hl
		ld	de,time_zone_act	;look for "CFG:1"
		ld	bc,13
		call	look_for_string
		push	af
		dec	(ix+6)
		ld	bc,7
		jp	z,error			;timeout
		pop	af
		jr	nz,activate_tz
		





read_t_again	ld	hl,(HLKO+1)
		push	hl
		ld	de,wait_ntp
		ld	hl,#4700
		call	PRINTHL
		pop	hl
		ld	(HLKO+1),hl

		ld	hl,at_cipsntptime
		call	send_string

		ld	bc,buf_len
		ld	hl,RX_buf		;now come the awaited data/body, accept it
		call	receive_data_timeout
		ld	bc,10			;seek for year 19xx
		or	a
		sbc	hl,bc
		ld	a,(hl)
		add	hl,bc
		cp	"1"

		push	af
		push	hl
		ld	hl,(HLKO+1)
		push	hl
		ld	de,spaces
		ld	hl,#4700
		call	PRINTHL
		pop	hl
		ld	(HLKO+1),hl
		pop	hl

		dec	(ix+6)
		jr	nz,rta1
		ld	hl,#4800
		ld	de,ntp_timeout
		call	PRINTHL
		ld	bc,2
		jp	error

rta1		pop	af			;CP "1"
		jr	z,read_t_again

						;year 2xxx found, continue


		ld	bc,6			;HL=length of incoming data
		or	a			;Z=all OK, received data (body) in RX buffer
		sbc	hl,bc
		ld	(hl),0			;at the end 0 have to come to print it


		ld	de,STATUS+6		;we seek for ":"
		ld	bc,20
		ld	hl,RX_buf
		call	look_for_string
		ld	bc,4
		jp	nz,error
						;now, current time&date is on DE

		push	de			;DE=begin of time#date

		ld	hl,#4800
		call	PRINTHL
						;print it

		pop	de
		push	de
		ex	de,hl			;now position zeros so you can start
		xor	a			;analysis
		ld	bc,3
		add	hl,bc
		ld	(hl),a
		inc	bc
		add	hl,bc
		ld	(hl),a
		dec	bc
		add	hl,bc
		ld	(hl),a
		add	hl,bc
		ld	(hl),a
		add	hl,bc
		ld	(hl),a
		add	hl,bc
		ld	(hl),a
		inc	bc
		inc	bc
		ld	(hl),a

		ld	bc,35
		ld	hl,days
		pop	de			;DE= start of date data retrieved by AT com.
		push	de
		call	look_for_string
		inc	de
		ld	a,(de)
		inc	de
		ld	(rtc_entry+3),a		;week days

		pop	de
		inc	de
		inc	de
		inc	de
		inc	de
		push	de
		ld	bc,5*12
		ld	hl,months
		call	look_for_string
		inc	de
		ld	a,(de)
		inc	de
		ld	(rtc_entry+5),a		;month

		pop	hl
		inc	hl
		inc	hl
		inc	hl
		inc	hl
		push	hl
		call	convASCII2hex
		ld	(rtc_entry+4),a		;days (0-31)

		pop	hl
		inc	hl
		inc	hl
		inc	hl
		push	hl
		call	convASCII2hex
		ld	(rtc_entry+2),a		;hours

		pop	hl
		inc	hl
		inc	hl
		inc	hl
		push	hl
		call	convASCII2hex
		ld	(rtc_entry+1),a		;mins

		pop	hl
		inc	hl
		inc	hl
		inc	hl
		push	hl
		call	convASCII2hex
		ld	(rtc_entry),a		;secs

		pop	hl
		inc	hl
		inc	hl
		inc	hl
		inc	hl
		inc	hl
		call	convASCII2hex
		ld	(rtc_entry+6),a		;year







		xor	a			;write a valid data to rtc
		ld	hl,rtc_entry
write_rtc_loop	ld	bc,port_rtc_select
		out	(c),a
		ld	e,(hl)
		ld	b,#77
		out	(c),e
		inc	hl
		inc	a
		cp	7
		jr	nz,write_rtc_loop
		
		jp	allok

				
days		db	"Mon",0,2,"Tue",0,3,"Wed",0,4,"Thu",0,5
		db	"Fri",0,6,"Sat",0,7,"Sun",0,1

months		db	"Jan",0,1,"Feb",0,2,"Mar",0,3,"Apr",0,4,"May",0,5,"Jun",0,6
		db	"Jul",0,7,"Aug",0,8,"Sep",0,9,"Oct",0,#10,"Nov",0,#11,"Dec",0,#12,0

rtc_entry	db	#55,#55,#55,#55,#55,#55,#55
			;secs, mins, hours, week day, day, month, year (all in hex)


/* 	;pro rucni input, zatim deaktivujeme
input		push	ix
		ld	ixh,3
		call	INPUT		;input 3 digit shift
		pop	ix		;valid range is <-11,13>

		cp	4
		ret	z		;user pressed CS+3, he aborted INPUT

		res	7,(ix+0)
		
		ld	de,INPBUFF
		ld	a,(de)
		cp	"-"
		jr	nz,not_minus
		inc	de
		set	7,(ix+0)

not_minus
		ld	a,(de)
nm1		cp	#30
		jr	c,inp_again
		cp	#39
		jr	nc,inp_again	
		inc	de
		ld	a,(de)
		cp	#20
		jr	nz,nm1


		bit	7,(ix+0)
		push	af
		ld	de,INPBUFF
		jr	z,eval_plus1
		inc	de

eval_plus1	call	ConvRStr16

		pop	af
		ld	a,l
		jr	z,eval_plus

		cp	12
		jr	c,gmt_range_ok
		jr	inp_again


eval_plus	cp	14
		jr	c,gmt_range_ok

inp_again	jr	input		;dopnit text

		

gmt_range_ok	ld	hl,gmt_shift
		push	hl
		ld	b,4
zero_gmt_range	ld	(hl),0
		inc	hl
		djnz	zero_gmt_range

		ld	hl,INPBUFF	;nyni se musi obsah INPBUFF naldirovat k prikazu
		pop	de		;DE=gmt_shift
			ld	bc,4
ldi_gmt_range	ldi
		ld	a,(hl)
		or	a
		jr	nz,ldi_gmt_range

;zde bude ret do hlavn rutiny
		

		jr	stack_pointer+3	
*/

block_asci	ds	5		;blocks converted to ASCII, (used by progress display)
		db	0







error		ld	h,b
		ld	l,c
		ld	de,block_asci
		push	de
		call	HL2ASCII
		ex	de,hl
		ld	(hl),0

		ld	hl,#4B00
		ld	de,error_txt
		call	PRINTHL
		pop	de
		call	PRINT

allok		ld	bc,0
		push	bc

end1		call	inkey
		or	a
		jr	nz,end1		;NZ= key is still pressed
					;Z= key is released

		ld	bc,#783B	;return CPU speed
		ld	a,#30
		out	(c),a
		inc	b
return_speed	ld	a,#55		;speed is 0-3
		out	(c),a

		pop	bc
stack_pointer	ld	sp,#5555
		exx
		ld	hl,10072
		exx
		ei
		ret		;return to BASIC, for the moment







;****************************************************************
;****************************************************************
;****************************************************************


;PARSING
;look for a string in received bytes string, for receive string evaluation purposes
;IN=HL start of the memory area, DE=start of the string we look for, must end with 0, BC=max.len of searched memory, OUT=Z=found, HL=start address of found string in memory, DE=byte coming after found string, NZ=string not found
;10)
look_for_string	ld (lfs5+1),sp
		set 0,(ix+0)

lfs3		ld a,(de)
		cp (hl)
		jr z,lfs_match

		bit 0,(ix+0)		;NZ=no match, was already matching?
		inc hl			;new
		jr nz,lfs2
		dec hl			;new
		pop de			;Z=was matching, now not anymore, renew looked for string
		pop de			;DE=start of searched string
		inc bc
lfs2		set 0,(ix+0)
;		inc hl			;new


lfs1		dec bc
		ld a,b
		or c
		jr nz,lfs3
		inc c			;BC max length reached, set NZ & return
		jr lfs5	

		



lfs_match	bit 0,(ix+0)
		jr z,lfs_alrea_match
		res 0,(ix+0)		;Z=match found, push address
		push de
		push hl

lfs_alrea_match	inc hl
		inc de
		ld a,(de)
		or a
		jr nz,lfs1		;continue macthing if string continues

		ld d,h
		ld e,l			;DE=byte coming after found string
		pop hl			;HL=start address of found string in memory

lfs4		bit 0,(ix+0)
lfs5		ld sp,#5555
		ret			;Z/NZ






;IN=HL=start of looked for string, must end with 0
;OUT=Z=string found, NZ=string not found
;11)
look_on_fly	exx
		ld	bc,0
		exx
		ld	e,#FF		;e= flag, Z=found,NZ=not found
		ld	b,e
		ld	c,e		;BC=timeout
lof3		push	hl
lof1		dec	bc		;bugfix 0.3
lof5		scf
		call	rece_byte
		jr	nz,lof4

		exx			;Z= no byte to receive. ESP can just breathe out and then continue
		dec	bc
		ld	a,c
		or	b
		exx
		jr	z,lofend	;Z=really, no byte to collect
		jr	lof5

lof4		cp	(hl)
		ld	e,0
		jr	z,lof2		

		dec	e		;NZ,char does not match, bring another byte
		pop	hl
		ld	a,b		;max of 65535 unsucessfull tries
		or	c
		jr	z,lofend1
		jr	lof3		

lof2		inc	hl
		ld	a,(hl)
		or	a
		jr	nz,lof1		;still not end, continue search
		
		
lofend		pop	hl
lofend1		inc	e
		dec	e
		ret			;Z/NZ




;RECEIVING
;IN=BC=max.length of data to receive, HL=start of received data, OUT=Z= desired length of data received,opravit vystupni flagy Z/NZ nyn neureny

;7)
receive_data_timeout
		ld de,#290		;timeout, if #80, only app. first #500 bytes carried correctly
receive_data1	ld a,b			;bios1.2, timeout have to increase to #200
		or c
		ret z			;Z=length reached, return
		scf
		call rece_byte
		jr nz,rd1
		dec de			;Z=no byte to receive
		ld a,e			;DE=timeout, in case there is receive interuption
		or d
		ret z
		jr receive_data1	;do not decrease BC, as no data came in

rd1		ld (hl),a
		inc hl
		dec bc
		jr receive_data1



;SENDING
;IN=HL string (ends with 0), OUT=Z=string has been sent
;1)
send_string	push hl			;print it first
		ld d,h
		ld e,l
		call PRINT
		pop hl

ss2		ld a,(hl)
		or a
		ret z			;Z=end of string

ss1		scf
		call send_byte
		jr nc,ss3
		push af
		ld a,3
		out (254),a		;magenta border= TX line is busy.....
		pop af			;maybe some timeout could be added here
		jr ss1

ss3		inc hl
		xor a
		out (254),a
		jr ss2


	IFUSED send_data

;IN=HL start of data, BC=data len, OUT=Z=data has been sent
;2)
send_data	ld a,b
		or c
		ret z			;Z=len of data in BC sent
		ld a,(hl)
		or a
		ret z
sd1		scf
;		ld e,a
		call send_byte
;		ld a,e
		jr c,sd1
		inc hl
		dec bc
		jr send_data

	ENDIF


;OTHERS
	
;16)
ConvRStr16
;===============================================================
;http://z80-heaven.wikidot.com/math#toc32
;Input: 
;     DE points to the base 10 number string in RAM. 
;Outputs: 
;     HL is the 16-bit value of the number 
;     DE points to the byte after the number 
;     BC is HL/10 
;     z flag reset (nz)
;     c flag reset (nc)
;Destroys: 
;     A (actually, add 30h and you get the ending token) 
;Size:  23 bytes 
;Speed: 104n+42+11c
;       n is the number of digits 
;       c is at most n-2 
;       at most 595 cycles for any 16-bit decimal value 
;===============================================================

     ld hl,0          ;  10 : 210000 
ConvLoop:             ; 
     ld a,(de)        ;   7 : 1A 
     sub 30h          ;   7 : D630 
     cp 10            ;   7 : FE0A 
     ret nc           ;5|11 : D0 
     inc de           ;   6 : 13 
                      ; 
     ld b,h           ;   4 : 44 
     ld c,l           ;   4 : 4D 
     add hl,hl        ;  11 : 29 
     add hl,hl        ;  11 : 29 
     add hl,bc        ;  11 : 09 
     add hl,hl        ;  11 : 29 
                      ; 
     add a,l          ;   4 : 85 
     ld l,a           ;   4 : 6F 
     jr nc,ConvLoop   ;12|23: 30EE 
     inc h            ; --- : 24 			
     jr ConvLoop      ; --- : 18EB



;17)
;IN=HL=number, OUT=DE=address with the string
HL2ASCII		res 7,(IX+0)		;Leading zero suppression
HL2ASCII_00000		ld bc,10000
			call string_dl
			ld bc,1000
			call string_dl
			ld bc,100
			call string_dl
			ld bc,10
			call string_dl
			ld a,l
			add a,'0'
			LD (DE),A
			INC DE
			RET
			
string_dl:		ld a,'0'-1

string_lp2:		inc a
			or a
			sbc hl,bc
			jr nc,string_lp2
			add hl,bc
			CP '0'
			JR Z,string_store

			SET 7,(IX+0)		;Had something other than zero

string_store:		BIT 7,(IX+0)		;Test if still leading zeros
			RET Z						
			LD (DE),A		;If not store it	
			INC DE
			RET	






	IFUSED wait_long
;18)
wait_long	ld	b,3		;wait long time, DE delay fixed (3x 65536 times)
wait_long_def	call	wait_short	;wait long time, DE delay suer defined (Bx 65536 times), IN=B=delay
		djnz	wait_long_def
		ret

wait_short	ld	de,0		;wait short time, DE delay fixed (65536 times) (tady by mela stacit polovicni hodnota tedy #8000)!!!!
wait_short_def	dec	de		;wait short time, DE delay user defined,IN=DE=delay
		ld	a,d
		or	e
		jr	nz,wait_short_def
		ret

	ENDIF





;29)
send_at_check_ok
		ld	hl,at
		call	send_string
		ld	bc,28
		jp	nz,error
;		ld	de,#100			;when it is away, it speeds up the search significantly
;		call	wait_short_def
		ld	hl,RX_buf
		ld	bc,20
		push	hl
		push	bc
		ld	de,RX_buf+1		;clear RX_buf before reading bytes to it
		ld	(hl),0
		ldir
		pop	bc
		pop	hl
		push	hl
		push	bc
		call	receive_data_timeout	;much quicker than look_on_fly
		pop	bc
		pop	hl
		ld	de,ok
		jp	look_for_string



;40) OUT CY 0=OK, 1= speed not supported
set_print_speed	scf
print_speed	call	set_baud
		push	af
		ld	a,d
ps2		ld	de,TX_buf
		push	de
		ld	h,0
		ld	l,a
		call	HL2ASCII
		ld	a,32
		ld	(de),a
		inc	de
		ld	(de),a
		xor	a
		inc	de
		ld	(de),a
		pop	de
		ld	hl,(HLKO+1)
		push	hl
		ld	hl,speed_pos+6
;		ld	(HLKO+1),hl
		call	PRINTHL
		pop	hl			;return print position
		ld	(HLKO+1),hl
		pop	af			;CY/NC
		ret



;48)
convASCII2hex
;IN=HL ASCII number address (2 digits) eg. hl,14
;OUT=A=#14
		ld	a,(hl)
		and	%00001111
		sla	a
		sla	a
		sla	a
		sla	a
		ld	e,a
		inc	hl
		ld	a,(hl)
		and	%00001111
		or	e
		ret



;commands
at		db "AT",#D,#A,0
ate0		db "ATE0",#D,#A,0
cipmux		db "AT+CIPMUX=",0
cipstatus	db "AT+CIPSTATUS",#D,#A,0
uart		db "AT+UART_CUR="
_1152000	db "1152000"
_3456000	db "3456000"	
_8100		db ",8,1,0,0",#D,#A
at_cwmode	db "AT+CWMODE=1",#D,#A,0	
at_cipsntpcfg?	db "AT+CIPSNTPCFG?",#D,#A,0
at_cipsntpcfg	db "AT+CIPSNTPCFG=1,",0
sntp_servers	db 44,34,"tik.cesnet.cz",34,44,34
		db "time.nist.gov",34,44,34,"cz.pool.ntp.org",34,#D,#A,0
at_cipsntptime	db "AT+CIPSNTPTIME?",#D,#A,0
at_rst		db "AT+RST",#D,#A,0


;answers
STATUS		db "STATUS:",0
answ_5		db "5",0
sendok		db "SEND "
ok		db "OK"
eol		db #D,#A,0
dvojt		db ":",0
ok_		db "OK",0
time_zone_act	db "CFG:1",0

;texts
wifi_nc_ap	db "not connected to AP",0
wifi_no_success	db "not successful",0
speed		db "Speed:",0
gmt_shift	db "GMT shift: ",0
neg_wifi	db "Negotiating Wifi connection... ",0
spaces		db "                               ",0
wait_ntp	db "Waiting for server...",0
ntp_timeout	db "NTP server timeout. Try again.",0
wifi_reset	db "Resetting Wifi...",0
disconnect	db "DISCONNECT",0
title		db "RTC settings via Wifi. Version 0.4, Hood, March_2021",0
title_len	equ $-title-1
error_txt	db "error=",0

;flags=IX+0
flag1		db 8		;see start of the code, 2=SSL connection,0=TCP
par_err		db #55		;IX+1= parity error counter
		db #FF		;IX+2= maximum number of parity errors
flag2		db 0		;IX+3, print send command,0=NO, 1=YES
;flag2		db 1		;IX+3
		db #20		;IX+4, maximum number of raw data reread, see "rece_loop"
		db 0		;IX+5, starting speed for bios
		db #55		;IX+6, timeout for initi wifi speed management (used on more places)
		db #55		;IX+7, uart speed currently set in HW
		db #55		;IX+8, maximum speed for bios (counted from FW)
		db #55		;IX+9, =buffer length counter for upload
;end of flags

core		db #55,#55,#55	

RX_buf		ds #80
TX_buf		ds #80


	include "keyboard_and_print.asm"

bios_start		
	incbin	"WifiBios16.cod"
bios_len	equ $-bios_start